==> linkit_sharepoint-1.0.0.tar.gz: linkit_sharepoint/src/Controller/SharepointController.php <==
<?php

namespace Drupal\linkit_sharepoint\Controller;

use Drupal\Component\Utility\Html;
use Drupal\Core\Controller\ControllerBase;
use Office365\Runtime\Auth\NetworkCredentialContext;
use Office365\SharePoint\ClientContext;
use Office365\Runtime\Http\RequestOptions;
use Office365\SharePoint\ListItem;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;

/**
 * Sharepoint controller class.
 *
 * @package Drupal\linkit_sharepoint\Controller
 */
class SharepointController extends ControllerBase {

  /**
   * The module configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   * @todo refactor into service class and controller class
   * @todo use proper dependency injection.
   */
  protected $config;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * Sharepoint controller constructor.
   */
  public function __construct() {
    $this->config = \Drupal::service('config.factory')
      ->get('linkit_sharepoint.adminsettings');
    $this->currentUser = \Drupal::service('current_user');
  }

  /**
   * Searches for a given keyword and returns SP ItemList.
   *
   * @param string $search_key
   *   The search string.
   *
   * @return mixed
   *   The result items.
   */
  public function search($search_key) {
    $this->getContext($ctx, $list);
    // Filter $search_key for bad characters.
    $search_key = preg_replace("/[^a-zA-Z0-9äöüÄÖÜ]+/", "", $search_key);
    $items = $list->getItems()
      ->select('*,EncodedAbsUrl')
      ->filter('substringof(\'' . $search_key . '\',TaxCatchAll/Term) or substringof(\'' . $search_key . '\',Title)');
    $ctx->load($items);
    $ctx->executeQuery();
    return $items->getData();
  }

  /**
   * Menu callback for autocompletion.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response containing the autocomplete suggestions for Views tags.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function autocompleteDocument(Request $request) {
    $matches = [];
    $search_key = $request->query->get('q');

    $sp = new SharepointController();
    $items = $sp->search($search_key);
    foreach ($items as $item) {
      $matches[] = [
        'value' => (int) $item->Id,
        'label' => Html::escape($item->Title) . ' (' . (int) $item->Id . ')',
      ];
    }
    return new JsonResponse($matches);
  }

  /**
   * Gets the Office365 ClientContext necessary for all SP calls.
   *
   * @param \Office365\SharePoint\ClientContext|null $ctx
   *   The client context object.
   * @param \Office365\SharePoint\SPList|null $list
   *   The SPList object.
   */
  private function getContext(&$ctx, &$list) {
    $url = $this->config->get('url');
    $userName = $this->config->get('user');
    $password = $this->config->get('password');
    $listTitle = $this->config->get('list_title');
    $authCtx = new NetworkCredentialContext($userName, $password);
    // CURLAUTH_BASIC; Default auth schema.
    $authCtx->AuthType = CURLAUTH_NTLM;
    // Initialize REST client.
    $ctx = new ClientContext($url, $authCtx);
    $web = $ctx->getWeb();
    $list = $web->getLists()->getByTitle($listTitle);
  }

  /**
   * Given a SP ListItem, get the SP/AD usernames which may read the item.
   *
   * @param \Office365\SharePoint\ListItem $listItem
   *   A sharepoint list item.
   *
   * @return array
   *   An array of usernames with read access.
   *
   * @throws \Exception
   */
  private function getReadUsers(ListItem $listItem) {
    $outArr = $assignments = $bindings = [];
    /** @var \Office365\SharePoint\RoleAssignment $assignment */
    foreach ($listItem->getRoleAssignments()->getIterator() as $assignment) {
      $assignments[] = $assignment;
      /** @var \Office365\SharePoint\RoleDefinition $binding */
      foreach ($assignment->getRoleDefinitionBindings()->getIterator() as $binding) {
        $bindings[$assignment->getPrincipalId()][] = $binding->getName();

        // Available permissions from Sharepoint are currently
        // 'Full Control', 'Limited Access', 'Lesen', 'Bearbeiten'.
        if ($binding->getName() === 'Lesen'
          || $binding->getName() === 'Full Control'
          || $binding->getName() === 'Bearbeiten') {

          $login_name = $assignment->getMember()->getLoginName();
          $startPos = strpos($login_name, '\\');
          // We strip unnecessary domain information.
          if ($startPos > 0) {
            $outArr[] = substr($login_name, $startPos + 1);
          }
          else {
            $outArr[] = $login_name;
          }
        }
      }
    }
    return $outArr;
  }

  /**
   * Tests if the configuration works.
   *
   * @return array
   *   A render array.
   */
  public function testconnection() {
    // Can we reach the server?
    $urlArr = parse_url($this->config->get('url'));
    $serverName = $urlArr['host'];
    if ($serverName == gethostbyname($serverName)) {
      return [
        '#markup' => 'Error: Cannot resolve the server name "' . $serverName . '". Maybe add i. e., 10.80.24.44 srv-80-24-44.dwezed.local to /etc/hosts.',
      ];
    }

    // Are all necessary properties set in the configuration?
    foreach (['list_title', 'user', 'password'] as $t) {
      if (!$this->config->get($t)) {
        return [
          '#markup' => 'Error: Missing property "' . $t . '" in the configuration.',
        ];
      }
    }

    try {
      $this->getContext($ctx, $list);
      $items = $list->getItems()->select('*,EncodedAbsUrl')->expand('Roleassignments/Member,RoleAssignments/RoleDefinitionBindings');
      $ctx->load($items);
      $ctx->executeQuery();
      $result = $items->getData();

      if (!is_array($result)) {
        return [
          '#markup' => 'Error: The list does not contain any items.',
        ];
      }

      $out = [];
      foreach ($result as $r) {
        $out[] = $r->Id . ' ' . $r->EncodedAbsUrl . ' readable by: ' . implode(', ', $this->getReadUsers($r));
      }

      return [
        '#markup' => 'The following documents can be read (there might be more): <br>' . implode('<br>', $out),
      ];

    }
    catch (\Exception $e) {
      $message = $e->getMessage();
      if ($message) {
        $message = '"' . $e->getMessage() . '"<br><br>Hint: Does the technical user have the rights to access role assignments?';
      }
      else {
        $message = 'Are the user credentials of the technical user correct?';
      }
      return [
        '#markup' => 'Error: ' . $message,
      ];
    }
  }

  /**
   * Document delivery and access check.
   *
   * Given the numeric ID of an SP document, this checks if the user may
   * access it. If yes, it sends the document to the user without saving the
   * file locally.
   *
   * @param int $id
   *   The Sharepoint document id.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response object.
   *
   * @throws \Exception
   */
  public function deliver($id) {
    $this->getContext($ctx, $list);
    $items = $list->getItems()->select('*,EncodedAbsUrl')->expand('Roleassignments/Member,RoleAssignments/RoleDefinitionBindings')->filter('Id eq ' . $id);
    // Save a query to retrieve list items from the server.
    $ctx->load($items);
    // Submit query to SharePoint Online REST service.
    $ctx->executeQuery();

    // We cannot access this object collection like an array, so we do it
    // this way.
    foreach ($items->getData() as $item) {
      break;
    }

    // ID did not exist; there was no item in the list.
    if (!isset($item)) {
      throw new UnauthorizedHttpException('');
    }

    // Check if user may read this document.
    $doDeliver = FALSE;
    // Array(drole), i. e., authenticated oder administrator.
    $droles = $this->currentUser->getRoles();
    // Array('drole', 'srole').
    $roleassignments = $this->config->get('roleassignments');
    // Array(srole).
    $sroles = $this->getReadUsers($item);
    foreach ($droles as $drole) {
      foreach ($roleassignments as $roleassignment) {
        if ($roleassignment['drole'] == $drole) {
          if (in_array($roleassignment['srole'], $sroles)) {
            $doDeliver = TRUE;
            break 2;
          }
        }
      }
    }
    if (!$doDeliver) {
      throw new UnauthorizedHttpException('');
    }

    $options = new RequestOptions($item->EncodedAbsUrl);
    // Follow file relocations?
    // $options->FollowLocation = TRUE.
    $data = $ctx->executeQueryDirect($options);

    // Construct binary response w/o saving the file locally.
    $response = new Response();
    $response->setContent($data->getContent());

    // We assume the data is usually binary.
    $response->headers->set('Content-Type', 'application/octet-stream');

    // Get only the filename.
    $filenameArr = explode('/', $item->EncodedAbsUrl);

    // Send the file with its original filename if present.
    $dispositionHeader = $response->headers->makeDisposition(
      ResponseHeaderBag::DISPOSITION_ATTACHMENT,
      urldecode($filenameArr[count($filenameArr) - 1]),
      'unkown.filename');
    $response->headers->set('Content-Disposition', $dispositionHeader);

    return $response;
  }

}

==> linkit_sharepoint-1.0.1.tar.gz: linkit_sharepoint/src/Controller/SharepointController.php <==
<?php

namespace Drupal\linkit_sharepoint\Controller;

use Drupal\Component\Utility\Html;
use Drupal\Core\Controller\ControllerBase;
use Office365\Runtime\Auth\NetworkCredentialContext;
use Office365\SharePoint\ClientContext;
use Office365\Runtime\Http\RequestOptions;
use Office365\SharePoint\ListItem;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;

/**
 * Sharepoint controller class.
 *
 * @package Drupal\linkit_sharepoint\Controller
 */
class SharepointController extends ControllerBase {

  /**
   * The module configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * The Sharepoint service.
   *
   * @var \Drupal\linkit_sharepoint\Sharepoint
   */
  protected $sharepoint;

  /**
   * SharepointController constructor.
   */
  public function __construct() {
    $this->config = \Drupal::service('config.factory')
      ->get('linkit_sharepoint.adminsettings');
    $this->currentUser = \Drupal::service('current_user');
    $this->sharepoint = \Drupal::service('linkit_sharepoint.sharepoint');
  }

  /**
   * Searches for a given keyword and returns SP ItemList.
   *
   * @param string $search_key
   *   The search string.
   *
   * @return mixed
   *   The result items.
   *
   * @deprecated
   */
  public function search($search_key) {
    $this->getContext($ctx, $list);
    // Filter $search_key for bad characters.
    $search_key = preg_replace("/[^a-zA-Z0-9äöüÄÖÜ]+/", "", $search_key);
    $items = $list->getItems()
      ->select('*,EncodedAbsUrl')
      ->filter('substringof(\'' . $search_key . '\',TaxCatchAll/Term) or substringof(\'' . $search_key . '\',Title)');
    $ctx->load($items);
    $ctx->executeQuery();
    return $items->getData();
  }

  /**
   * Menu callback for autocompletion.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response containing the autocomplete suggestions for Views tags.
   */
  public function autocompleteDocument(Request $request) {
    $matches = [];
    $search_key = $request->query->get('q');
    $items = $this->sharepoint->search($search_key);
    foreach ($items as $item) {
      $matches[] = [
        'value' => (int) $item->Id,
        'label' => Html::escape($item->Title) . ' (' . (int) $item->Id . ')',
      ];
    }
    return new JsonResponse($matches);
  }

  /**
   * Gets the Office365 ClientContext necessary for all SP calls.
   *
   * @param \Office365\SharePoint\ClientContext|null $ctx
   *   The client context object.
   * @param \Office365\SharePoint\SPList|null $list
   *   The SPList object.
   *
   * @deprecated
   */
  private function getContext(&$ctx, &$list) {
    $url = $this->config->get('url');
    $userName = $this->config->get('user');
    $password = $this->config->get('password');
    $listTitle = $this->config->get('list_title');

    $authCtx = new NetworkCredentialContext($userName, $password);
    // CURLAUTH_BASIC; Default auth schema.
    $authCtx->AuthType = CURLAUTH_NTLM;
    // Initialize REST client.
    $ctx = new ClientContext($url, $authCtx);
    $web = $ctx->getWeb();
    $list = $web->getLists()->getByTitle($listTitle);
  }

  /**
   * Given a SP ListItem, get the SP/AD usernames which may read the item.
   *
   * @param \Office365\SharePoint\ListItem $listItem
   *   A sharepoint list item.
   *
   * @return array
   *   An array of usernames with read access.
   *
   * @throws \Exception
   *
   * @deprecated
   */
  private function getReadUsers(ListItem $listItem) {
    $outArr = $assignments = $bindings = [];
    /** @var \Office365\SharePoint\RoleAssignment $assignment */
    foreach ($listItem->getRoleAssignments()->getIterator() as $assignment) {
      $assignments[] = $assignment;
      /** @var \Office365\SharePoint\RoleDefinition $binding */
      foreach ($assignment->getRoleDefinitionBindings()->getIterator() as $binding) {
        $bindings[$assignment->getPrincipalId()][] = $binding->getName();

        // Available permissions from Sharepoint are currently
        // 'Full Control', 'Limited Access', 'Lesen', 'Bearbeiten'.
        if ($binding->getName() === 'Lesen'
          || $binding->getName() === 'Full Control'
          || $binding->getName() === 'Bearbeiten') {

          $login_name = $assignment->getMember()->getLoginName();
          $startPos = strpos($login_name, '\\');
          // We strip unnecessary domain information.
          if ($startPos > 0) {
            $outArr[] = substr($login_name, $startPos + 1);
          }
          else {
            $outArr[] = $login_name;
          }
        }
      }
    }
    return $outArr;
  }

  /**
   * Tests if the configuration works.
   *
   * @return array
   *   A render array.
   */
  public function testconnection() {
    // Can we reach the server?
    $urlArr = parse_url($this->config->get('url'));
    $serverName = $urlArr['host'];
    if ($serverName == gethostbyname($serverName)) {
      return [
        '#markup' => 'Error: Cannot resolve the server name "' . $serverName . '". Maybe add i. e., 10.80.24.44 srv-80-24-44.dwezed.local to /etc/hosts.',
      ];
    }

    // Are all necessary properties set in the configuration?
    foreach (['list_title', 'user', 'password'] as $t) {
      if (!$this->config->get($t)) {
        return [
          '#markup' => 'Error: Missing property "' . $t . '" in the configuration.',
        ];
      }
    }

    try {
      $this->getContext($ctx, $list);
      $items = $list->getItems()->select('*,EncodedAbsUrl')->expand('Roleassignments/Member,RoleAssignments/RoleDefinitionBindings');
      $ctx->load($items);
      $ctx->executeQuery();
      $result = $items->getData();

      if (!is_array($result)) {
        return [
          '#markup' => 'Error: The list does not contain any items.',
        ];
      }

      $out = [];
      foreach ($result as $r) {
        $out[] = $r->Id . ' ' . $r->EncodedAbsUrl . ' readable by: ' . implode(', ', $this->getReadUsers($r));
      }

      return [
        '#markup' => 'The following documents can be read (there might be more): <br>' . implode('<br>', $out),
      ];

    }
    catch (\Exception $e) {
      $message = $e->getMessage();
      if ($message) {
        $message = '"' . $e->getMessage() . '"<br><br>Hint: Does the technical user have the rights to access role assignments?';
      }
      else {
        $message = 'Are the user credentials of the technical user correct?';
      }
      return [
        '#markup' => 'Error: ' . $message,
      ];
    }
  }

  /**
   * Document delivery and access check.
   *
   * Given the numeric ID of an SP document, this checks if the user may
   * access it. If yes, it sends the document to the user without saving the
   * file locally.
   *
   * @param int $id
   *   The Sharepoint document id.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response object.
   *
   * @throws \Exception
   */
  public function deliver($id) {
    $this->sharepoint->initializeContext();
    $items = $this->sharepoint->list->getItems()->select('*,EncodedAbsUrl')
      ->expand('Roleassignments/Member,RoleAssignments/RoleDefinitionBindings')
      ->filter('Id eq ' . $id);

    // Save a query to retrieve list items from the server.
    $this->sharepoint->context->load($items);
    // Submit query to SharePoint Online REST service.
    $this->sharepoint->context->executeQuery();

    // We cannot access this object collection like an array, so we do it
    // this way.
    foreach ($items->getData() as $item) {
      break;
    }

    // ID did not exist; there was no item in the list.
    if (!isset($item)) {
      throw new UnauthorizedHttpException('');
    }

    // Check if user may read this document.
    $doDeliver = FALSE;
    // Array(drole), i. e., authenticated oder administrator.
    $droles = $this->currentUser->getRoles();
    // Array('drole', 'srole').
    $roleassignments = $this->config->get('roleassignments');
    // Array(srole).
    $sroles = $this->sharepoint->getReadUsers($item);
    foreach ($droles as $drole) {
      foreach ($roleassignments as $roleassignment) {
        if ($roleassignment['drole'] == $drole) {
          if (in_array($roleassignment['srole'], $sroles)) {
            $doDeliver = TRUE;
            break 2;
          }
        }
      }
    }
    if (!$doDeliver) {
      throw new UnauthorizedHttpException('');
    }

    $options = new RequestOptions($item->EncodedAbsUrl);
    // @todo Follow file relocations?
    // $options->FollowLocation = TRUE.
    $data = $this->sharepoint->context->executeQueryDirect($options);

    // Construct binary response w/o saving the file locally.
    $response = new Response();
    $response->setContent($data->getContent());

    // We assume the data is usually binary.
    $response->headers->set('Content-Type', 'application/octet-stream');

    // Get only the filename.
    $filenameArr = explode('/', $item->EncodedAbsUrl);

    // Send the file with its original filename if present.
    $dispositionHeader = $response->headers->makeDisposition(
      ResponseHeaderBag::DISPOSITION_ATTACHMENT,
      urldecode($filenameArr[count($filenameArr) - 1]),
      'unkown.filename');
    $response->headers->set('Content-Disposition', $dispositionHeader);

    return $response;
  }

}
